@@ -1,6 +1,6 @@
 package Spreadsheet::ParseXLSX;
 our $AUTHORITY = 'cpan:DOY';
-$Spreadsheet::ParseXLSX::VERSION = '0.21'; # TRIAL
+$Spreadsheet::ParseXLSX::VERSION = '0.22'; # TRIAL
 use strict;
 use warnings;
 use 5.008;
@@ -166,7 +166,7 @@ sub _parse_sheet {
     $sheet->{MaxCol} = -1;
     $sheet->{Selection} = [ 0, 0 ];
 
-    my @merged_cells;
+    my %merged_cells;
 
     my @column_formats;
     my @column_widths;
@@ -177,6 +177,8 @@ sub _parse_sheet {
     my $default_row_height   = 15;
     my $default_column_width = 10;
 
+    my %cells;
+
     my $sheet_xml = $self->_new_twig(
         twig_roots => {
             #XXX need a fallback here, the dimension tag is optional
@@ -252,7 +254,7 @@ sub _parse_sheet {
                     ];
                     for my $row ($toprow .. $bottomrow) {
                         for my $col ($leftcol .. $rightcol) {
-                            push(@merged_cells, [$row, $col]);
+                            $merged_cells{"$row;$col"} = 1;
                         }
                     }
                 }
@@ -317,16 +319,6 @@ sub _parse_sheet {
                 $twig->purge;
             },
 
-        }
-    );
-
-    $sheet_xml->parse( $sheet_file );
-
-    # 2nd pass: cell/row building is dependent on having parsed the merge definitions
-    # beforehand.
-
-    $sheet_xml = $self->_new_twig(
-        twig_roots => {
             's:sheetData/s:row' => sub {
                 my ( $twig, $row_elt ) = @_;
 
@@ -381,9 +373,6 @@ sub _parse_sheet {
                     my $format_idx = $cell->att('s') || 0;
                     my $format = $sheet->{_Book}{Format}[$format_idx];
                     die "unknown format $format_idx" unless $format;
-                    $format->{Merged} = !!grep {
-                        $row == $_->[0] && $col == $_->[1]
-                    } @merged_cells;
 
                     # see the list of built-in formats below in _parse_styles
                     # XXX probably should figure this out from the actual format string,
@@ -392,31 +381,36 @@ sub _parse_sheet {
                         $long_type = 'Date';
                     }
 
+                    my $formula = $cell->first_child('s:f');
                     my $cell = Spreadsheet::ParseExcel::Cell->new(
                         Val      => $val,
                         Type     => $long_type,
-                        Merged   => $format->{Merged},
+                        Merged   => undef, # fix up later
                         Format   => $format,
                         FormatNo => $format_idx,
-                        ($cell->first_child('s:f')
-                            ? (Formula => $cell->first_child('s:f')->text)
+                        ($formula
+                            ? (Formula => $formula->text)
                             : ()),
                         Rich     => $Rich,
                     );
                     $cell->{_Value} = $sheet->{_Book}{FmtClass}->ValFmt(
                         $cell, $sheet->{_Book}
                     );
+                    $cells{"$row;$col"} = $cell;
                     $sheet->{Cells}[$row][$col] = $cell;
                 }
 
                 $twig->purge;
             },
-
         }
     );
 
     $sheet_xml->parse( $sheet_file );
 
+    for my $key (keys %merged_cells) {
+        $cells{$key}{Merged} = 1 if $cells{$key};
+    }
+
     if ( ! $sheet->{Cells} ){
         $sheet->{MaxRow} = $sheet->{MaxCol} = -1;
     }
@@ -832,18 +826,15 @@ sub _parse_styles {
             %ignore,
         );
 
-        if (!$ignore{IgnoreFont}) {
-            $opts{FontNo} = 0+$xml_fmt->att('fontId');
-        }
-        if (!$ignore{IgnoreFill}) {
-            $opts{Fill} = $fills[$xml_fmt->att('fillId')],
-        }
-        if (!$ignore{IgnoreBorder}) {
-            $opts{BdrStyle} = $borders[$xml_fmt->att('borderId')]{styles},
-            $opts{BdrColor} = $borders[$xml_fmt->att('borderId')]{colors},
-            $opts{BdrDiag}  = $borders[$xml_fmt->att('borderId')]{diagonal},
-        }
-        if (!$ignore{IgnoreAlignment} && $alignment) {
+        $opts{FmtIdx}   = 0+($xml_fmt->att('numFmtId')||0);
+        $opts{FontNo}   = 0+($xml_fmt->att('fontId')||0);
+        $opts{Font}     = $font[$opts{FontNo}];
+        $opts{Fill}     = $fills[$xml_fmt->att('fillId')||0];
+        $opts{BdrStyle} = $borders[$xml_fmt->att('borderId')||0]{styles};
+        $opts{BdrColor} = $borders[$xml_fmt->att('borderId')||0]{colors};
+        $opts{BdrDiag}  = $borders[$xml_fmt->att('borderId')||0]{diagonal};
+
+        if ($alignment) {
             $opts{AlignH} = $halign{$alignment->att('horizontal') || 'general'};
             $opts{Wrap}   = $self->_xml_boolean($alignment->att('wrapText'));
             $opts{AlignV} = $valign{$alignment->att('vertical') || 'bottom'};
@@ -852,18 +843,14 @@ sub _parse_styles {
             $opts{Shrink} = $self->_xml_boolean($alignment->att('shrinkToFit'));
             # JustLast => $iJustL,
         }
-        if (!$ignore{IgnoreNumberFormat}) {
-            $opts{FmtIdx} = 0+$xml_fmt->att('numFmtId');
-        }
-        if (!$ignore{IgnoreProtection} && $protection) {
+
+        if ($protection) {
             $opts{Lock} = defined $protection->att('locked')
                 ? $self->_xml_boolean($protection->att('locked'))
-                : 1,
+                : 1;
             $opts{Hidden} = $self->_xml_boolean($protection->att('hidden'));
         }
 
-        $opts{Font} = $font[$opts{FontNo}];
-
         # Style    => $iStyle,
         # Key123   => $i123,
         # Merge   => $iMerge,
@@ -1106,7 +1093,7 @@ Spreadsheet::ParseXLSX - parse XLSX files
 
 =head1 VERSION
 
-version 0.21
+version 0.22
 
 =head1 SYNOPSIS
 
